
import { localize } from '../../../../nls.js';
import { Color } from '../../../../base/common/color.js';
import { registerColor, transparent } from '../colorUtils.js';
import { focusBorder, contrastBorder } from './baseColors.js';

registerColor('sash.hoverBorder', focusBorder, ( localize(2202, "Border color of active sashes.")));
const badgeBackground = registerColor('badge.background', { dark: '#4D4D4D', light: '#C4C4C4', hcDark: Color.black, hcLight: '#0F4A85' }, ( localize(
 2203,
 "Badge background color. Badges are small information labels, e.g. for search results count."
)));
const badgeForeground = registerColor('badge.foreground', { dark: Color.white, light: '#333', hcDark: Color.white, hcLight: Color.white }, ( localize(
 2204,
 "Badge foreground color. Badges are small information labels, e.g. for search results count."
)));
const activityWarningBadgeForeground = registerColor('activityWarningBadge.foreground', { dark: ( Color.black.lighten(0.2)), light: Color.white, hcDark: null, hcLight: ( Color.black.lighten(0.2)) }, ( localize(2205, 'Foreground color of the warning activity badge')));
const activityWarningBadgeBackground = registerColor('activityWarningBadge.background', { dark: '#CCA700', light: '#BF8803', hcDark: null, hcLight: '#CCA700' }, ( localize(2206, 'Background color of the warning activity badge')));
const activityErrorBadgeForeground = registerColor('activityErrorBadge.foreground', { dark: ( Color.black.lighten(0.2)), light: Color.white, hcDark: null, hcLight: ( Color.black.lighten(0.2)) }, ( localize(2207, 'Foreground color of the error activity badge')));
const activityErrorBadgeBackground = registerColor('activityErrorBadge.background', { dark: '#F14C4C', light: '#E51400', hcDark: null, hcLight: '#F14C4C' }, ( localize(2208, 'Background color of the error activity badge')));
const scrollbarShadow = registerColor('scrollbar.shadow', { dark: '#000000', light: '#DDDDDD', hcDark: null, hcLight: null }, ( localize(2209, "Scrollbar shadow to indicate that the view is scrolled.")));
const scrollbarSliderBackground = registerColor('scrollbarSlider.background', { dark: ( ( Color.fromHex('#797979')).transparent(0.4)), light: ( ( Color.fromHex('#646464')).transparent(0.4)), hcDark: ( transparent(contrastBorder, 0.6)), hcLight: ( transparent(contrastBorder, 0.4)) }, ( localize(2210, "Scrollbar slider background color.")));
const scrollbarSliderHoverBackground = registerColor('scrollbarSlider.hoverBackground', { dark: ( ( Color.fromHex('#646464')).transparent(0.7)), light: ( ( Color.fromHex('#646464')).transparent(0.7)), hcDark: ( transparent(contrastBorder, 0.8)), hcLight: ( transparent(contrastBorder, 0.8)) }, ( localize(2211, "Scrollbar slider background color when hovering.")));
const scrollbarSliderActiveBackground = registerColor('scrollbarSlider.activeBackground', { dark: ( ( Color.fromHex('#BFBFBF')).transparent(0.4)), light: ( ( Color.fromHex('#000000')).transparent(0.6)), hcDark: contrastBorder, hcLight: contrastBorder }, ( localize(2212, "Scrollbar slider background color when clicked on.")));
const progressBarBackground = registerColor('progressBar.background', { dark: ( Color.fromHex('#0E70C0')), light: ( Color.fromHex('#0E70C0')), hcDark: contrastBorder, hcLight: contrastBorder }, ( localize(
 2213,
 "Background color of the progress bar that can show for long running operations."
)));
const chartLine = registerColor('chart.line', { dark: '#236B8E', light: '#236B8E', hcDark: '#236B8E', hcLight: '#236B8E' }, ( localize(2214, "Line color for the chart.")));
const chartAxis = registerColor('chart.axis', { dark: ( ( Color.fromHex('#BFBFBF')).transparent(0.4)), light: ( ( Color.fromHex('#000000')).transparent(0.6)), hcDark: contrastBorder, hcLight: contrastBorder }, ( localize(2215, "Axis color for the chart.")));
const chartGuide = registerColor('chart.guide', { dark: ( ( Color.fromHex('#BFBFBF')).transparent(0.2)), light: ( ( Color.fromHex('#000000')).transparent(0.2)), hcDark: contrastBorder, hcLight: contrastBorder }, ( localize(2216, "Guide line for the chart.")));

export { activityErrorBadgeBackground, activityErrorBadgeForeground, activityWarningBadgeBackground, activityWarningBadgeForeground, badgeBackground, badgeForeground, chartAxis, chartGuide, chartLine, progressBarBackground, scrollbarShadow, scrollbarSliderActiveBackground, scrollbarSliderBackground, scrollbarSliderHoverBackground };
